import React from 'react';
import { Input } from 'antd';
import styles from './styles.less';
import { antiAssign } from '@/apollo-table/utils/utils';
import { ApolloInputProps } from '../editInterface';
import CellContainer from '../container';

const ApolloInput = (props: ApolloInputProps) => {
    const { maxLength, onChange, value, style } = props;
    const selfProps = antiAssign(props, ['columnConfig', 'onChange', 'style']);
    const changeValue = (e) => {
        if (typeof onChange === 'function') {
            onChange(e.target.value);
        }
    };

    const newStyle: any = {
        ...style,
    };
    if (maxLength && style) {
        newStyle.minHeight = Number(style.minHeight) + 10;
        newStyle.paddingBottom = 18;
    }
    return (
        <div className={styles.container}>
            <Input className={styles.input} style={newStyle} {...selfProps} onChange={changeValue} />
            {!!maxLength && (
                <span className={styles.wordNumber}>
                    已输入{(value || '').length}/{maxLength}
                </span>
            )}
        </div>
    );
};
export default CellContainer(ApolloInput);
